/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef AUTOCONFIGURATOR_HPP
#define AUTOCONFIGURATOR_HPP

/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/

#include <string>
#include <map>
#include <vector>
using namespace std;

#include <boost/thread/detail/singleton.hpp>
using namespace boost::detail::thread;


class Program
{
public:
Program(string _name, string _cmd):name(_name),cmd(_cmd){}
Program(){}

	string getCmd() const
	{
	    return cmd;
	}

	string getName() const
	{
	    return name;
	}

	void setCmd(const string& theValue)
	{
	    cmd = theValue;
	}

	void setName(const string& theValue)
	{
	    name = theValue;
	}
	
	bool filled()
	{
		return (cmd!="" && name!="");
	}
	
	
private:
string name;
string cmd;
};


class AutoConfigurator
{
public:
	
	bool isFirstRun();
	void firstRunConfiguration();
	void clearOldConfiguration();
	void assignProgramsToMime(map<string,vector<Program> > &mimeProgramsMap);
	void setDefaultPrograms(map<string,Program> mimePrograms,map <string, vector<Program> > mimeProgramsMap);
private:
	
	AutoConfigurator();	
	~AutoConfigurator();
	
		
	const string userDirectory;
	string globalConfigurationFile;
	string mimeToProgramFile;
	bool firstRun;

friend class singleton<AutoConfigurator>;

};

#endif
