/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef COMMAND_HPP
#define COMMAND_HPP

#include "FileList.hpp"
#include "Cancelable.hpp"
#include "CommandInfo.hpp"

#include <boost/thread/recursive_mutex.hpp>
#include <boost/thread/mutex.hpp>
#include<boost/tokenizer.hpp>
using namespace boost;


class FileList;

/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/
class Command : public Cancelable
{
public:
    virtual void init(FileList *_filelist,string _name,CommandInfo _cmdInfo);
    virtual void execute()=0;
    virtual ~Command(){}
    virtual bool isThread(){return false;}
    void setParam ( const string& theValue );


	void setFileList ( FileList* theValue )
	{
		fileList = theValue;
	}

	FileList* getFileList() const
	{
		return fileList;
	}

	void setExclusive ( bool theValue )
	{
		exclusive = theValue;
	}
	

	bool getExclusive() const
	{
		return exclusive;
	}

	virtual void setCanceledExecution ()
	{
		canceledExecution = true;
	}
	

	virtual bool isCanceledExecution()
	{
		return canceledExecution;
	}

	string getParam() const
	{
		return param;
	}
	
	string getParam(string name)
	{
		return params[name];
	}
	
	
	
	
    
protected:
	FileList *fileList;
	string name;
	bool exclusive;
	volatile bool canceledExecution;
	string param;
	CommandInfo cmdInfo;
	
private:	
	boost::mutex muteX; 
	map<string,string> params;
};

class CommandOpposite : public Command
{
public:
    virtual void init(FileList *_filelist,string _name, CommandInfo _cmdInfo);
    virtual ~CommandOpposite();

protected:
	FileList *fileListOpposite;

};

#endif
