/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "CommandsManager.hpp"
#include "ThreadCommand.hpp"
/*
#include "commands/CmdCreateDir.hpp"
#include "commands/CmdCopy.hpp"
#include "commands/CmdExternal.hpp"
#include "commands/CmdPythonPlugin.hpp"
*/
#include "Settings.hpp"
#include "FileListSync.hpp"
#include "FilesEngine.hpp"

#include "libfactory/Factorable.hpp"
#include "libfactory/ClassFactory.hpp"
#include "binreloc.h"


CommandsManager::~CommandsManager()
{
}

Command * CommandsManager::getCommand( string name, FileList *fileList,string param )
{
	Command *cmd=NULL;
	
	try
	{
		Settings& settings=singleton<Settings>::instance();
		
		CommandInfo cmdInfo=settings.getCommand( name );
		
		ClassFactory &factory=singleton<ClassFactory>::instance();
		
		if(cmdInfo.getType()=="alias")
		{
			param=cmdInfo.getParam();
			string alias=cmdInfo.getAlias();
			cmdInfo=settings.getCommand( alias );
			name=cmdInfo.getName();
		
		
		}
		
		
		
		if(cmdInfo.getType()=="external")
		{
			cmd=dynamic_cast<Command*>(factory.createPure( "CmdExternal" ));	
		}
		else if(cmdInfo.getType()=="python plugin")
		{
			cmd=dynamic_cast<Command*>(factory.createPure( "CmdPythonPlugin" ));		
		}
		else if(cmdInfo.getType()=="builtin")
		{	
			if(cmdInfo.getLibName().size())
			{
				cmd=dynamic_cast<Command*>(factory.createPure( "Cmd"+name,cmdInfo.getLibName() ));	
			}
			else
			{
				cmd=dynamic_cast<Command*>(factory.createPure( "Cmd"+name ));
			}
		}
		
		
		if(cmd)
		cmd->init( fileList,name,cmdInfo);
		
		if(param!="")
		cmd->setParam(param);
		
	
		if(cmd && cmdInfo.getType()=="python plugin" || cmdInfo.getType()=="builtin" || (cmdInfo.getType()=="external" && fileList->getVfsType()!="Local") || cmdInfo.getForceReload())//multithread
		{
			if(cmdInfo.getRunAsThread())
			{
			#ifdef DEBUG
			cout<<"running command as thread:"<<name<<endl;
			#endif
				FileList *fl=fileList->getFileListSync();
				cmd->setFileList(fl);
				
				
					/*FIXME*/
					if(name =="StopThreads")
					{
						cmd->setExclusive(true);
					}
				cmd=new ThreadCommand(cmd);
			}
		}
	}
	catch(...)
	{
		cerr<<"error loading command: "<<name<<endl;
	}
		
	return cmd;
}

CommandsManager::CommandsManager( )
{

	ClassFactory &factory=singleton<ClassFactory>::instance();
	
	if(FilesEngine::getEnv("NAO_VFS_PLUGINS_PATH")!="")
	factory.addBaseDirectory( FilesEngine::getEnv("NAO_VFS_PLUGINS_PATH") );
	
	if(FilesEngine::getEnv("NAO_COMMANDS_PLUGINS_PATH")!="")
	factory.addBaseDirectory( FilesEngine::getEnv("NAO_COMMANDS_PLUGINS_PATH") );
	
	factory.addBaseDirectory(string(br_find_lib_dir(PATH_LIBDIR))+"/nao");


}


