/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef FILELIST_H
#define FILELIST_H

#include "config.h"

#include "FileList.hpp"
#include "FileListSync.hpp"
#include "Vfs.hpp"
#include "VfsSync.hpp"
#include "FileListIcons.hpp"
#include "Panel.hpp"
#include "CommandsManager.hpp"
#include "Settings.hpp"
#include "FileIcon.hpp"
#include "Mime.hpp"
#include "ThreadCommand.hpp"
#include <string>
#include <vector>
#include <list>

#ifdef HAVE_LIBMAGIC
	#include <magic.h>
#endif


class CommandsManager;


struct FileModificationTime
{
	path fileName;
	time_t modificationTime;

};


/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/
class FileListImpl: public FileList
{
public:
	FileListImpl():settings(singleton<Settings>::instance()){}
	FileListImpl(FileListIcons *fli,Panel *panel);	
	~FileListImpl();	
	void openParentDir();
	void openIcon( FileIcon fileIcon );
	void openDirectory( path p);
	void reloadDirectory();
	
	/*
	void addFile(string name);
	void removeFile(string name);	
	File getFile(int nr);
	
	void clearFiles();
	void refreshFile(File file);
	int getFilePosition(path source);
	int getNumberOfFiles();	
	*/
	
	
	vector<path> getSelectedFileIconsPaths(void);
	vector<string> getSelectedFileIcons(void);
	FileIcon getCurrentFileIcon();
	FileIcon getFileIcon(string p);
	void addFileIcon(File file);
	void setFiles(vector<File> filesVec);
	void addFile(string p);
	void replaceFileIcon(FileIcon fileIcon);
	void removeFileIcon(string p);
	void removeFileIcons();
	void selectFileIcon(string p);
	vector<string>getAllFilesPaths();
	void refreshFileIcon(string p);
	
	
	
	
	
	vector<string> getDirectories(void);
 	void runCommand(string name,string param="");
	vector<string> getToolbarCommands(string); 
	vector<string> filterContexCommands(vector<string>);  
	Vfs* getVfs();
	FileListIcons* getFileListIcons() { return fileListIcons;	}
	void setOpposite(FileList* theValue);
	void sort(int nr=-1);	
	void textFieldExecute(string);
	void textFieldCommandSwitch(int);
	void clicked(void);
	void startThreadProcessing(ThreadCommand * tc);
	void stopThreadProcessing(ThreadCommand *tc);
	void waitForThreads();	
	vector<string> getAvailableCommandsForSelectedFiles();
	path accessFile(path filePath);
	void releaseFile(path filePath, bool commit=true);
	void acquire();
	void release();
	void tick();
	
	
	bool isAcquired()
	{
		if(acquiredNum>0)
		return true;
		else
		return false;
	}
	
	path getCurrentDirectory()
	{
	    return currentDirectory;
	}

	FileList* getOpposite() 
	{
	    	return opposite;
	}

	void setDroppedFileNames(const vector< string >& theValue)
	{
	    dragAndDrop=true;
	    droppedFileNames = theValue;
	}

	bool isDragAndDrop()
	{
	    return dragAndDrop;
	}

	void setDragAndDrop(bool theValue)
	{
	    dragAndDrop = theValue;
	}

	string getTextFieldData()
	{
	    return textFieldData;
	}

	string getVfsName()
	{
	    return vfsName;
	}


	void setFileListSync ( FileList* theValue )
	{
		fileListSync = theValue;
	}
	

	FileList* getFileListSync()
	{
		if(fileListSync==NULL)
			fileListSync=new FileListSync(this);
		return fileListSync;
	}

	void setCurrentDirectory ( const path& theValue );

	path getNewDirectory()
	{
		return newDirectory;
	}

	vector< string >& getPathHistory()
	{
		return pathHistory;
	}

	void setVfsType ( const string& theValue )
	{
		vfsType = theValue;
	}
	

	string getVfsType()
	{
		return vfsType;
	}
	
	

private:
		
	
	void openFile(path p);
	FileIcon createFileIcon(File file);
	string getCommandForFile(path p);
	
	
	string vfsName;
	string vfsType;
	Vfs *vfs;
	path currentDirectory;
	path newDirectory;
	FileListIcons *fileListIcons;
	Panel *panel;
	
	vector<FileIcon>files;
	vector<FileIcon>::iterator filesIter;
	map<string,int> filesMap;
	map<string,int>::iterator filesMapIter;
	
	
	FileList *opposite;
	Settings &settings;
	int sortNr;
	bool ascending;
	vector<string>headers;
	vector<string>droppedFileNames;
	map<ThreadCommand*,ThreadCommand*> threadCommands;
	bool dragAndDrop;
	string textFieldCommandName;
	string textFieldData;
	vector <string>textFieldCommands;
	Mime mime;
	FileList *fileListSync;	
	Vfs *vfsSync;
	bool acquired;
	vector<string> pathHistory;
	
	bool stopAllThreads;
	int threadsNumber;
	bool isLocalVfs;

	map<path,FileModificationTime>tmpFiles;
	
	int acquiredNum;
		
	#ifdef HAVE_LIBMAGIC
	static magic_set* ms;
	#endif


};



#endif
