/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "FileListSync.hpp"

void FileListSync::openDirectory(path p)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->openDirectory(p);
}

void FileListSync::openParentDir()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
    	f->openParentDir();
}

void FileListSync::openIcon(FileIcon fileIcon)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->openIcon(fileIcon);
}

void FileListSync::reloadDirectory()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->reloadDirectory();
}

vector< string > FileListSync::getDirectories(void )
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getDirectories();
}

void FileListSync::setOpposite(FileList * theValue)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->setOpposite(theValue);
}

FileListIcons * FileListSync::getFileListIcons()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getFileListIcons();
}

Vfs * FileListSync::getVfs()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getVfs();
}

vector< string > FileListSync::filterContexCommands(vector< string >commands )
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->filterContexCommands(commands);
}

vector< string > FileListSync::getToolbarCommands(string toolbar)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getToolbarCommands(toolbar);
}

void FileListSync::runCommand(string name,string param)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->runCommand(name,param);
}

path FileListSync::getCurrentDirectory()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getCurrentDirectory();
}

void FileListSync::clicked(void )
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->clicked();
}

void FileListSync::textFieldCommandSwitch(int n)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->textFieldCommandSwitch(n);
}

void FileListSync::textFieldExecute(string text)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->textFieldExecute(text);
}

FileList * FileListSync::getOpposite()
{
	/*TODO CHCECK THIS*/
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getOpposite();
}

void FileListSync::setDroppedFileNames(const vector< string > & theValue)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->setDroppedFileNames(theValue);
}

bool FileListSync::isDragAndDrop()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->isDragAndDrop();
}

void FileListSync::setDragAndDrop(bool theValue)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->setDragAndDrop(theValue);
}

string FileListSync::getTextFieldData()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getTextFieldData();
}

string FileListSync::getVfsType()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getVfsType();
}

void FileListSync::sort(int nr)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->sort(nr);
}

void FileListSync::setFileListSync(FileList * theValue)
{
}

FileList * FileListSync::getFileListSync()
{
	return this;
}

void FileListSync::stopThreadProcessing(ThreadCommand * tc)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->stopThreadProcessing(tc);
}

void FileListSync::startThreadProcessing(ThreadCommand * tc)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->startThreadProcessing(tc);
}

void FileListSync::setCurrentDirectory(const path & theValue)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->setCurrentDirectory(theValue);
}

path FileListSync::getNewDirectory()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getNewDirectory();

}
/*only one thread can execute this - exclusive*/
void FileListSync::waitForThreads()
{
	//boost::recursive_mutex::scoped_lock lock(muteX);
	f->waitForThreads();

} 

vector< string > & FileListSync::getPathHistory()
{
	/*not safe, only one thread can operate on this list at the same time*/
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getPathHistory();
}

string FileListSync::getVfsName()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getVfsName();
}


vector< string > FileListSync::getAvailableCommandsForSelectedFiles()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getAvailableCommandsForSelectedFiles();

}

path FileListSync::accessFile(path filePath)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->accessFile(filePath);
}

void FileListSync::releaseFile(path filePath, bool commit)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->releaseFile(filePath,commit);
}

void FileListSync::refreshFileIcon(string p)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->refreshFileIcon(p);
}

vector< string > FileListSync::getAllFilesPaths()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getAllFilesPaths();
}

void FileListSync::selectFileIcon(string p)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->selectFileIcon(p);
}

void FileListSync::removeFileIcons()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->removeFileIcons();
}

void FileListSync::removeFileIcon(string p)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->removeFileIcon(p);
}

void FileListSync::replaceFileIcon(FileIcon fileIcon)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->replaceFileIcon(fileIcon);
}

void FileListSync::addFileIcon(File file)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->addFileIcon(file);
}

FileIcon FileListSync::getFileIcon(string p)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getFileIcon(p);
}

vector< path > FileListSync::getSelectedFileIconsPaths(void )
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getSelectedFileIconsPaths();
}

void FileListSync::addFile(string p)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->addFile(p);
}


void FileListSync::setFiles(vector< File > filesVec)
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->setFiles(filesVec);
}


FileIcon FileListSync::getCurrentFileIcon()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getCurrentFileIcon();
}

void FileListSync::acquire()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->acquire();
}

void FileListSync::release()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	f->release();
}

bool FileListSync::isAcquired()
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->isAcquired();
}

void FileListSync::tick()
{
	//boost::recursive_mutex::scoped_lock lock(muteX);
	f->tick();
}

vector< string > FileListSync::getSelectedFileIcons(void )
{
	boost::recursive_mutex::scoped_lock lock(muteX);
	return f->getSelectedFileIcons();
}


