/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "Settings.hpp"

/*TEMPORARY*/
#include <iostream>
#include <fstream>
using namespace std;

#include <boost/lexical_cast.hpp>
using boost::lexical_cast;

#include "FilesEngine.hpp"
#include "vfs/VfsLocal.hpp"
#include "binreloc.h"
#include "config.h"

Settings::Settings()
{

	string path_cfg=br_find_etc_dir(PATH_CFG);
	if(path_cfg=="/usr/etc")
	path_cfg="/etc";
	
	path file = FilesEngine::getHomeDir( );
	file = file / ".nao/nao.conf";
	
	if ( !FilesEngine::exists ( file ))
	{
		file =  path(path_cfg) / path("nao.conf");
	}
	
	xml.loadXml(file);
	xml.setXmlFileName(FilesEngine::getHomeDir( )/".nao/nao.conf");

	version=xml.read("/NaoConfig/version");
	
	nonUTF8DefaultCharset=xml.read("/NaoConfig/non_utf8_default_charset");
	


	if(version!=VERSION)
	{
		xml.clear();		
		file =  path(path_cfg) / path("nao.conf");
		xml.loadXml(file);
		xml.setXmlFileName(FilesEngine::getHomeDir( )/".nao/nao.conf");
		versionMismatch=true;
		version=VERSION;
	}


	//programPath=xml.read("/NaoConfig/path");
	dataPath=string(br_find_data_dir(PATH_DATADIR))+"/nao";
	

	
	iconsTheme=xml.read("/NaoConfig/icons_theme");
	
	windowWidth = atoi ( xml.read( "/NaoConfig/mainwindow/width" ).c_str () );
	windowHeight= atoi ( xml.read( "/NaoConfig/mainwindow/height" ).c_str () );
	
	filesHistorySize= atoi ( xml.read( "/NaoConfig/files_history_size" ).c_str () );
	
	defaultShutter=xml.read("/NaoConfig/default_shutter");

	showHiddenFiles=string2bool(xml.read("/NaoConfig/show_hidden_files"));
	saveWindowSize=string2bool(xml.read("/NaoConfig/save_window_size"));
	
	markInaccessibleFolders=string2bool(xml.read("/NaoConfig/mark_inaccessible_folders"));
	markInaccessibleFiles=string2bool(xml.read("/NaoConfig/mark_inaccessible_files"));
	saveOriginalCharsetOnRename=string2bool(xml.read("/NaoConfig/save_original_charset_on_rename"));
	createNewFileWithUTF8=string2bool(xml.read("/NaoConfig/create_new_file_with_utf8"));
	
	startupPlacement=xml.read( "/NaoConfig/startup_placement" );
	
	singlePanel=true;
	if(xml.read( "/NaoConfig/panels" )=="double")
	singlePanel=false;
	
	if (xml.openPath("/NaoConfig/commands/command"))
	{
		map<string,string>readMap;
        	while ( xml.readNode( readMap ) )
        	{
              		string name=readMap["name"];
              		string type=readMap["type"];
              		string key=readMap["key"];
              		string keyMask=readMap["key_mask"];
              		string thread=readMap["thread"];
              		string param=readMap["param"];
              		
              		map<string,string> descriptionLanguages;
              		
              		map<string,string>::iterator iter;
              		for(iter=readMap.begin();iter!=readMap.end();++iter)
              		{
              			string name=iter->first;
              			if(name.size()==14)
              			{
              				if(name.substr(0,11)=="description")
              				{
              					string lang=name.substr(12,2);
              					descriptionLanguages[lang]=iter->second;
              				}
              			}
              		}
              		
              		//string description=readMap["description"];
              		
              		
              		
              		string forceReload=readMap["force_reload"];
              		string gesture=readMap["gesture"];
              		string needSelectedFile=readMap["need_selected_file"];             		
              		             		
              		             		
        		CommandInfo cmd(name,type,readMap["icon"]);
        		
        		cmd.setDescription(descriptionLanguages);
        		
        		if(gesture!="")
              		cmd.setGesture(gesture);
              		
       			cmd.setNeedSelectedFile(string2bool(needSelectedFile));               		
              		cmd.setForceReload(string2bool(forceReload));
       
        		
        		/*
              		if(description!="")
              		cmd.setDescription(description);
        		else
        		cmd.setDescription(name);
        		*/
        		
        		if(type=="external" ||type=="python plugin")
        		cmd.setExec(readMap["exec"]);
        		
        		if(thread=="no")
        			cmd.setRunAsThread(false);
        		else
        			cmd.setRunAsThread(true);
        		
        		if(type=="alias")
        		{
        			cmd.setAlias(readMap["alias"]);
        		}
        		
        		cmd.setParam(param);
        		
        		if(key!="")
        		{
        		cmd.setKey(key);
        		cmd.setKeyMask(keyMask);
        		}
        		        		 		
        		if(type=="builtin")
        		{
        			string libName=readMap["lib"];
        			if(libName.size())
        				cmd.setLibName( libName );
        		}
        		
        		commands[name]=cmd;
        		commandsByNumber[cmd.getNr()]=cmd;
        		//allCommands.push_back(name);
        		readMap.clear();
        	}
	
	}
	

	
	if (xml.openPath("/NaoConfig/file_types/mime"))
	{
		map<string,vector<string> > node;
		
        	while ( xml.readNodeVector( node ) )
        	{
              		string name=node["name"].at(0);	
              		string defaultCommand;              		
              		string icon;
        		string color;
        		string backColor;
        		
        		MimeInfo mimeInfo(name);
        		
              		try{
              		defaultCommand=node["default"].at(0);	
              		mimeInfo.setDefaultCommand(defaultCommand);
              		}catch(...){}
              		try{
              		icon=node["icon"].at(0);
              		mimeInfo.setIcon(icon);
              		}catch(...){}	
              		try{
              		color=node["color"].at(0);
              		mimeInfo.setColor(color);
              		}catch(...){}
              		try{
              		backColor=node["backcolor"].at(0);
              		mimeInfo.setBackColor(backColor);
              		}catch(...){}
              		
              		              		
              		mimeInfo.setCommands(node["command"]);          		
       		
        		mime[name]=mimeInfo;
        		node.clear();
        		
        	}
	
	}
	
	fontDescriptions["normal"]=xml.read("/NaoConfig/fonts/normal");
	fontDescriptions["small"]=xml.read("/NaoConfig/fonts/small");
	fontDescriptions["big"]=xml.read("/NaoConfig/fonts/big");
	
	typingSpeed = atoi(xml.read("/NaoConfig/speed_delay/typing_speed").c_str());
	clickSpeed = atoi(xml.read("/NaoConfig/speed_delay/click_speed").c_str());
	scrollSpeed = atoi(xml.read("/NaoConfig/speed_delay/scroll_speed").c_str());
	scrollDelay = atoi(xml.read("/NaoConfig/speed_delay/scroll_delay").c_str());
	blinkSpeed = atoi(xml.read("/NaoConfig/speed_delay/blink_speed").c_str());
	animSpeed = atoi(xml.read("/NaoConfig/speed_delay/anim_speed").c_str());
	menuPause = atoi(xml.read("/NaoConfig/speed_delay/menu_pause").c_str());
	tooltipPause = atoi(xml.read("/NaoConfig/speed_delay/tooltip_pause").c_str());
	tooltipTime = atoi(xml.read("/NaoConfig/speed_delay/tooltip_time").c_str());
	dragDelta = atoi(xml.read("/NaoConfig/speed_delay/drag_delta").c_str());
    	wheelLines = atoi(xml.read("/NaoConfig/speed_delay/wheel_lines").c_str());
	
	
	baseColor=xml.read("/NaoConfig/colors/base");
	borderColor=xml.read("/NaoConfig/colors/border");
	backColor=xml.read("/NaoConfig/colors/back");
	foreColor=xml.read("/NaoConfig/colors/fore");
	selbackColor=xml.read("/NaoConfig/colors/selback");
	selforeColor=xml.read("/NaoConfig/colors/selfore");
	tipbackColor=xml.read("/NaoConfig/colors/tipback");
	tipforeColor=xml.read("/NaoConfig/colors/tipfore");
	menubackColor=xml.read("/NaoConfig/colors/menuback");
	menuforeColor=xml.read("/NaoConfig/colors/menufore");
	hiliteColor=xml.read("/NaoConfig/colors/hilite");
	shadowColor=xml.read("/NaoConfig/colors/shadow");
	mainColor=xml.read("/NaoConfig/colors/main");
	
	
	/*TODO*/
	if (xml.openPath("/NaoConfig/toolbar/toolbar1/command"))
	{	
		string command;
		while ( xml.readNodeString( command ) )
        	{
        		toolbarCommands["toolbar1"].push_back(command);
        	}
	}
	
	if (xml.openPath("/NaoConfig/toolbar/toolbar2/command"))
	{	
		string command;
		while ( xml.readNodeString( command ) )
        	{
        		toolbarCommands["toolbar2"].push_back(command);
        	}
	}
	
	
	
	defaultPath=xml.read("/NaoConfig/default_path/path");
	defaultVfsName=xml.read("/NaoConfig/default_path/type");
	
	rightPath=xml.read("/NaoConfig/right_path/path");
	rightVfsName=xml.read("/NaoConfig/right_path/type");
	
	leftPath=xml.read("/NaoConfig/left_path/path");
	leftVfsName=xml.read("/NaoConfig/left_path/type");
	
	previewSize=atoi(xml.read("/NaoConfig/preview_size").c_str());
	maxThumbSize=atoi(xml.read("/NaoConfig/max_thumb_size").c_str());
	
	
	if (xml.openPath("/NaoConfig/shutters/shutter"))
	{	
		map<string,vector<string> > node;
		
		while ( xml.readNodeVector( node ) )
		{			
			shutters[node["name"].at(0)]=node["command"];
			node.clear();
		}
	}
	

	if (xml.openPath("/NaoConfig/virtual_file_systems/vfs"))
	{	
		map<string,vector<string> > node;
		
		while ( xml.readNodeVector( node ) )
		{		
			string name=node["name"].at(0);	
			vfsTypes[name]=node["type"].at(0);	
			vfsHeaders[name]=node["header"];
			vfsAvailableHeaders[name]=node["available_header"];
			vector<string>::iterator iter;	
			vector<int> vecInt;		
			for(iter=node["width"].begin();iter!=node["width"].end();++iter)
			{
				vecInt.push_back(lexical_cast<int>(*iter));
			}
			vfsHeadersWidth[name]=vecInt;
			vfsStyles[name]=node["style"].at(0);
			node.clear();
		}
	}
	
	if (xml.openPath("/NaoConfig/history/remote_vfs/hostnames/hostname"))
	{	
		string str;
		while ( xml.readNodeString( str ) )
		{
			historyRemoteVfsHostnames.push_back(str);
		}
	}
	
	if (xml.openPath("/NaoConfig/history/remote_vfs/users/user"))
	{	
		string str;
		while ( xml.readNodeString( str ) )
		{
			historyRemoteVfsUsers.push_back(str);
		}
	}
	
	
	if (xml.openPath("/NaoConfig/history/files_history/file"))
	{	
		string str;
		while ( xml.readNodeString( str ) )
		{
			filesHistory.push_back(str);
		}
	}
	
	
	if (xml.openPath("/NaoConfig/text_field_commands/command"))
	{	
		map<string,vector<string> > node;
		
		while ( xml.readNodeVector( node ) )
		{			
			textFieldCommands[node["name"].at(0)]=node["last"];
			node.clear();
		}
	}
	
	
	
	
    std::ifstream ifs( "/usr/share/mime/globs", ios::in );

    if ( ifs )
    {
	while ( !ifs.eof () )
	{
		std::string tmp;
		getline ( ifs, tmp );
	
		if ( tmp.empty() || tmp[ 0 ] == '#' )
		{
		continue;
		}
	
		std::string mime = tmp.substr( 0, tmp.find ( ":" ) );

		availableMime.insert(mime);
	}
    }
	
	
}


path Settings::getIconsDir( )
{
	return path(dataPath)/path("icons")/path(iconsTheme); 
}

vector< string > Settings::getToolbarCommands( string dock )
{
	return toolbarCommands[dock];
}

string Settings::getCommandIcon( string command )
{
	return commands[command].getIconName();
}

CommandInfo Settings::getCommand( string name )
{
	if(commands.find(name)!=commands.end())
		return commands[name];
	else
		return CommandInfo();	
}

CommandInfo Settings::getCommand(int nr)
{
	return commandsByNumber[nr];
}

void Settings::addSettingsObservator( SettingsObservator * observator )
{
	observators.push_back(observator);

}

void Settings::removeSettingsObservator( SettingsObservator * observator )
{
	observators.erase(find(observators.begin(),observators.end(),observator));
}


void Settings::notifyObservators( void )
{

	vector< SettingsObservator*>::iterator iter;
	for(iter=observators.begin();iter!=observators.end();++iter)
	{
		(*iter)->notify();
	}

}

Settings::~ Settings( )
{
	xml.clear();
	xml.addNodeString("/","NaoConfig");
	
	xml.addNodeString("/NaoConfig","version",version);
	xml.addNodeString("/NaoConfig","non_utf8_default_charset",nonUTF8DefaultCharset);
	

	xml.addNodeString("/NaoConfig","save_original_charset_on_rename",bool2string(saveOriginalCharsetOnRename));
	xml.addNodeString("/NaoConfig","create_new_file_with_utf8",bool2string(createNewFileWithUTF8));
	
	
	xml.addNodeString("/NaoConfig","mainwindow");
	xml.addNodeString("/NaoConfig/mainwindow","width",lexical_cast<string>(windowWidth));
	xml.addNodeString("/NaoConfig/mainwindow","height",lexical_cast<string>(windowHeight));
	
	xml.addNodeString("/NaoConfig","files_history_size",lexical_cast<string>(filesHistorySize));
	
	xml.addNodeString("/NaoConfig","startup_placement",startupPlacement);
	
	xml.addNodeString("/NaoConfig","default_shutter",defaultShutter);
	

	xml.addNodeString("/NaoConfig","show_hidden_files",bool2string(showHiddenFiles));
	xml.addNodeString("/NaoConfig","save_window_size",bool2string(saveWindowSize));

	xml.addNodeString("/NaoConfig","mark_inaccessible_folders",bool2string(markInaccessibleFolders));
	xml.addNodeString("/NaoConfig","mark_inaccessible_files",bool2string(markInaccessibleFiles));

	
	
	xml.addNodeString("/NaoConfig","speed_delay");
	
	xml.addNodeString("/NaoConfig/speed_delay","typing_speed",lexical_cast<string>(typingSpeed));
	xml.addNodeString("/NaoConfig/speed_delay","click_speed",lexical_cast<string>(clickSpeed));
	xml.addNodeString("/NaoConfig/speed_delay","scroll_speed",lexical_cast<string>(scrollSpeed));
	xml.addNodeString("/NaoConfig/speed_delay","scroll_delay",lexical_cast<string>(scrollDelay));
	xml.addNodeString("/NaoConfig/speed_delay","blink_speed",lexical_cast<string>(blinkSpeed));
	xml.addNodeString("/NaoConfig/speed_delay","anim_speed",lexical_cast<string>(animSpeed));
	xml.addNodeString("/NaoConfig/speed_delay","menu_pause",lexical_cast<string>(menuPause));
	xml.addNodeString("/NaoConfig/speed_delay","tooltip_pause",lexical_cast<string>(tooltipPause));
	xml.addNodeString("/NaoConfig/speed_delay","tooltip_time",lexical_cast<string>(tooltipTime));
	xml.addNodeString("/NaoConfig/speed_delay","drag_delta",lexical_cast<string>(dragDelta));
	xml.addNodeString("/NaoConfig/speed_delay","wheel_lines",lexical_cast<string>(wheelLines));

	xml.addNodeString("/NaoConfig","colors");
	
	xml.addNodeString("/NaoConfig/colors","base",baseColor);
	xml.addNodeString("/NaoConfig/colors","back",backColor);
	xml.addNodeString("/NaoConfig/colors","border",borderColor);
	xml.addNodeString("/NaoConfig/colors","fore",foreColor);
	xml.addNodeString("/NaoConfig/colors","selback",selbackColor);
	xml.addNodeString("/NaoConfig/colors","selfore",selforeColor);
	xml.addNodeString("/NaoConfig/colors","tipback",tipbackColor);
	xml.addNodeString("/NaoConfig/colors","tipfore",tipforeColor);
	xml.addNodeString("/NaoConfig/colors","menuback",menubackColor);
	xml.addNodeString("/NaoConfig/colors","menufore",menuforeColor);
	xml.addNodeString("/NaoConfig/colors","hilite",hiliteColor);
	xml.addNodeString("/NaoConfig/colors","shadow",shadowColor);
	xml.addNodeString("/NaoConfig/colors","main",mainColor);


	xml.addNodeString("/NaoConfig","icons_theme",iconsTheme);
	
	xml.addNodeString("/NaoConfig","default_path");
	xml.addNodeString("/NaoConfig/default_path","path",defaultPath);
	xml.addNodeString("/NaoConfig/default_path","type",defaultVfsName);
	
	xml.addNodeString("/NaoConfig","left_path");
	xml.addNodeString("/NaoConfig/left_path","path",leftPath);
	xml.addNodeString("/NaoConfig/left_path","type",leftVfsName);
	
	xml.addNodeString("/NaoConfig","right_path");
	xml.addNodeString("/NaoConfig/right_path","path",rightPath);
	xml.addNodeString("/NaoConfig/right_path","type",rightVfsName);	
	
	xml.addNodeString("/NaoConfig","fonts");
	xml.addNodeString("/NaoConfig/fonts","normal",fontDescriptions["normal"]);
	xml.addNodeString("/NaoConfig/fonts","big",fontDescriptions["big"]);
	xml.addNodeString("/NaoConfig/fonts","small",fontDescriptions["small"]);	
	
	xml.addNodeString("/NaoConfig","preview_size",lexical_cast<string>(previewSize));
	xml.addNodeString("/NaoConfig","max_thumb_size",lexical_cast<string>(maxThumbSize));
	
	

	if(singlePanel)
	xml.addNodeString( "/NaoConfig","panels","single");
	else
	xml.addNodeString( "/NaoConfig","panels","double");
	
	xml.addNodeString("/NaoConfig","toolbar");
		
	map<string,vector<string> >::iterator toolbarIter;	
	for(toolbarIter=toolbarCommands.begin();toolbarIter!=toolbarCommands.end();++toolbarIter)
	{
		xml.addNodeString("/NaoConfig/toolbar",toolbarIter->first);
	
		vector<string>::iterator vectorIter;	
		for(vectorIter=toolbarIter->second.begin();vectorIter!=toolbarIter->second.end();++vectorIter)
		xml.addNodeString("/NaoConfig/toolbar/"+toolbarIter->first,"command",*vectorIter);
		
	}	
		

	xml.addNodeString("/NaoConfig","shutters");
	map< string, vector<string> >::iterator shuttersIter;	
	for(shuttersIter=shutters.begin();shuttersIter!=shutters.end();++shuttersIter)
	{	
		map< string, vector<string> > shutterMap;
		vector<string>vec;
		vec.push_back(shuttersIter->first);
		shutterMap["name"]=vec;
		shutterMap["command"]=shuttersIter->second;
		xml.writeNodeVector("/NaoConfig/shutters","shutter", shutterMap);
	}
	
	xml.addNodeString("/NaoConfig","commands");	
	map<string,CommandInfo>::iterator commandsIter;	
	for(commandsIter=commands.begin();commandsIter!=commands.end();++commandsIter)
	{	
		xml.writeNode("/NaoConfig/commands","command",commandsIter->second.getMap() );
	}
	
	xml.addNodeString("/NaoConfig","file_types");
	map<string,MimeInfo>::iterator mimeIter;	
	for(mimeIter=mime.begin();mimeIter!=mime.end();++mimeIter)
	{	
		xml.writeNodeVector("/NaoConfig/file_types","mime",mimeIter->second.getMap() );
	}


	xml.addNodeString("/NaoConfig","virtual_file_systems");
	map< string, vector<string> >::iterator vfsIter;	
	for(vfsIter=vfsHeaders.begin();vfsIter!=vfsHeaders.end();++vfsIter)
	{	
		map< string, vector<string> > vfsMap;
		vector<string>vec;
		vec.push_back(vfsIter->first);
		vector<string>vec2;
		vec2.push_back(vfsStyles[vfsIter->first]);
		vfsMap["name"]=vec;
		
		vector<string>vec3;
		vec3.push_back(vfsTypes[vfsIter->first]);
		vfsMap["type"]=vec3;
		
		vfsMap["header"]=vfsIter->second;		
		vfsMap["style"]=vec2;
		
		vfsMap["available_header"]=vfsAvailableHeaders[vfsIter->first];
		
		vector<int> vecInt=vfsHeadersWidth[vfsIter->first];
		vector<int>::iterator iter;
		vector<string> vecString;
		for(iter=vecInt.begin();iter!=vecInt.end();++iter)
		{
			vecString.push_back(lexical_cast<string>(*iter));
		}
		
		vfsMap["width"]=vecString;
		
		xml.writeNodeVector("/NaoConfig/virtual_file_systems","vfs", vfsMap);
	}


	xml.addNodeString("/NaoConfig","text_field_commands");
	map< string, vector<string> >::iterator tfcIter;	
	for(tfcIter=textFieldCommands.begin();tfcIter!=textFieldCommands.end();++tfcIter)
	{	
		map< string, vector<string> > tfcMap;
		vector<string>vec;
		vec.push_back(tfcIter->first);
		tfcMap["name"]=vec;
		tfcMap["last"]=tfcIter->second;
		xml.writeNodeVector("/NaoConfig/text_field_commands","command", tfcMap);
	}

	xml.addNodeString("/NaoConfig","history");
	xml.addNodeString("/NaoConfig/history","remote_vfs");
	
	map< string, vector<string> > tfcMap;
	tfcMap["hostname"]=historyRemoteVfsHostnames;
	xml.writeNodeVector("/NaoConfig/history/remote_vfs","hostnames", tfcMap);

	map< string, vector<string> > tfcMap2;
	tfcMap2["user"]=historyRemoteVfsUsers;
	xml.writeNodeVector("/NaoConfig/history/remote_vfs","users", tfcMap2);
	
	
	xml.addNodeString("/NaoConfig/history","files_history");
	
	list<string>::iterator iter;
	
	for(iter=filesHistory.begin();iter!=filesHistory.end();++iter)
	{	
		xml.addNodeString("/NaoConfig/history/files_history","file",*iter);
	}

}


vector< string > Settings::getAllCommands( )
{
	vector<string> names;
	map<string,CommandInfo>::iterator iter;
	for(iter=commands.begin();iter!=commands.end();++iter)
	{
		names.push_back(iter->first);
	}
	
	return names;
}

void Settings::setCommand( CommandInfo cmd )
{
	commands[cmd.getName()]=cmd;
	commandsByNumber[cmd.getNr()]=cmd;
	/*TODO*/
//	notifyObservators();
}

vector< string > Settings::getHeaders( string Vfs )
{
	return vfsHeaders[Vfs];
}

MimeInfo Settings::getMime(string name)
{
	map<string,MimeInfo>::iterator mimeIterator;
	mimeIterator=mime.find(name);
	if(mimeIterator!=mime.end())
		return mimeIterator->second;
	else
		return MimeInfo::CreateEmpty(name);
}

vector< string > Settings::getShutters(void )
{
	vector<string> shuttersList;
	map< string, vector<string> >::iterator shuttersIter;	
	for(shuttersIter=shutters.begin();shuttersIter!=shutters.end();++shuttersIter)
	{	
		shuttersList.push_back(shuttersIter->first);
	}
	
	return shuttersList;
}

vector< string > Settings::getShutter(string name)
{
	return shutters[name];
}

path Settings::getXmlGuiPath(string xmlGuiFile)
{
	/*FIXME*/
	return path("/home/compbatant/xml_nao_gui.xml");
}

string Settings::getFileListStyle(string Vfs )
{
	return vfsStyles[Vfs];
}

void Settings::setFileListStyle(string Vfs, string style )
{
	vfsStyles[Vfs]=style;
}


string Settings::getSortHeader( string Vfs )
{
	/*FIXME*/
	return "name";
}



void Settings::setMime( MimeInfo mimeInfo )
{
	mime[mimeInfo.getName()]=mimeInfo;
}

string Settings::getFontDescription( string fontName )
{
	return fontDescriptions[fontName];	
}

void Settings::setFontDescription( string fontName, string description )
{
	fontDescriptions[fontName]=description;	
}

vector< string > Settings::getTextFieldLast( string command )
{
	return textFieldCommands[command];
}

vector< string > Settings::getTextFieldCommands( )
{
	vector<string>commands;
	map<string,vector<string> >::iterator iter;
	for(iter=textFieldCommands.begin();iter!=textFieldCommands.end();++iter)
	{
		commands.push_back(iter->first);
	}
	return commands;
	
}

string Settings::getMimeIcon( string name )
{

	map<string,MimeInfo>::iterator mimeIterator;
	mimeIterator=mime.find(name);
	if(mimeIterator!=mime.end())
		return mimeIterator->second.getIcon();
	else
		return "";
	
}

vector< int > Settings::getHeadersWidth( string vfs )
{
	return vfsHeadersWidth[vfs];
}

void Settings::setHeadersWidth( string Vfs, vector< int >vec )
{
	vfsHeadersWidth[Vfs]=vec;
}

string Settings::getVfsType(string name)
{
	return vfsTypes[name];
}

void Settings::removeCommand(string name)
{
	map<string,CommandInfo>::iterator iter;
	iter=commands.find(name);
	if(iter!=commands.end())
		commands.erase(iter);
}

void Settings::removeMime(string name)
{
	map<string,MimeInfo>::iterator iter;
	iter=mime.find(name);
	if(iter!=mime.end())
		mime.erase(iter);
}


vector< string > Settings::getAllMimeNames()
{
	vector<string> names;
	map<string,MimeInfo>::iterator iter;
	for(iter=mime.begin();iter!=mime.end();++iter)
	{
		names.push_back(iter->first);
	}
	
	return names;
}


vector< string > Settings::getAllVfs()
{
	
	vector<string> names;
	map<string,vector<string> >::iterator iter;
	for(iter=vfsHeaders.begin();iter!=vfsHeaders.end();++iter)
	{
		names.push_back(iter->first);
	}
	
	return names;
}

void Settings::setHeaders(string vfs, vector< string > headers)
{
	vfsHeaders[vfs]=headers;
}

vector< string > Settings::getAvailableHeaders(string vfs)
{
	return vfsAvailableHeaders[vfs];
}

vector< string > Settings::getAvailableIconsThemes()
{

	vector<string>themes;
	
	/*TODO ~/.nao/icons add*/
	VfsLocal vfs;
	path p=path(dataPath)/path("icons");
	Directory directory=vfs.openDir(p);	

		try
		{
			while(1)
			{
			File file=directory.getNextFile();	
			
				if(file.normal() && file.isDirectory())
				{
					themes.push_back(file.getName());		
				}
			
			}		
		}catch(...){}	
		
	return themes;	
}

void Settings::setShutter(string name, vector< string > commands)
{
	shutters[name]=commands;
}

void Settings::setToolbarCommands(string toolbar, vector< string > commands)
{
	toolbarCommands[toolbar]=commands;
}

vector< string > Settings::getToolbars()
{
	vector<string>names;
	
	map<string,vector<string> >::iterator iter;
	for(iter=toolbarCommands.begin();iter!=toolbarCommands.end();++iter)
	{
		names.push_back(iter->first);
	}
	
	return names;
}

void Settings::removeShutter(string name)
{
	
	map<string,vector<string> >::iterator iter;
	iter=shutters.find(name);
	if(iter!=shutters.end())
	{
		shutters.erase(iter);
	}
}

void Settings::addFilesHistory(string filePath)
{

	if(filesHistory.end()==find(filesHistory.begin(),filesHistory.end(),filePath))
	{	
		if(filesHistory.size()>filesHistorySize)
		filesHistory.pop_front();
	
		filesHistory.push_back(filePath);
	}
}

list< string > Settings::getFilesHistory()
{
	return filesHistory;
}



string Settings::getLanguage()
{
	#ifdef  HAVE_LIBFOX_1_6


	string lc_messages=FilesEngine::getEnv("LC_MESSAGES");
	if(lc_messages=="")
	lc_messages=FilesEngine::getEnv("LC_ALL");
	
	if(lc_messages=="")
	lc_messages=FilesEngine::getEnv("LANG");	
	
	if(lc_messages=="")
	return "en";
	
	string lang;
	
	lang=lc_messages.substr(0,lc_messages.find("_"));
	
	//cout<<"lang="<<lang<<endl;
	return lang;
	
	#else	
	return  "en";
	#endif
	
}


