/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef VFSBASE_HPP
#define VFSBASE_HPP

#include "Vfs.hpp"
#include "Mime.hpp"
#include "Settings.hpp"

/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/
class VfsBase : public Vfs
{
public:
	VfsBase(string type);
	~VfsBase();
	void copy(Vfs * vfs2,path source, path destination,Status * status, Cancelable *cancelable);
	path symlink(const path& file);   
	void init(Panel *panel,QuestionManager *questionManager); 
	double getTotalSize(path &p, Cancelable *cancelable);
	void chmodRecursive(path &source,unsigned int mode);
    	void remove(path &source);
    	void changeOwnerRecursive(path &source,string user);
    	void changeGroupRecursive(path &source,string group);
    	
    	void setFileSize(vector<string>&fields, File& file);
	void setFileOwner(vector<string>&fields, File& file);
	void setFileGroup(vector<string>&fields, File& file);
	void setFileMime(vector<string>&fields, File& file);
	void setFilePermissions(vector<string>&fields, File& file);
	void setAccessTime(vector<string>&fields, File& file);
	void setModificationTime(vector<string>&fields, File& file);
	void setCreationTime(vector<string>&fields, File& file);
	void setPath(vector<string>&fields, File& file);
	
	
	void createSymlink(path &source,path &destination);
	
	File readFullFile(File file);
	
	
	virtual vector<string> getUsers();
    	virtual vector<string> getGroups();
    	
    	
    	template<class T>
	void goRecursive(File & file, T & operation,bool visitChildrenFirst=false,Cancelable *cancelable=NULL);
    	  
protected:	

	void setHeaders(vector<string>headers);
	
	vector<void (VfsBase::*)( vector<string>&, File& )> fileInfoVector;
	vector<void (VfsBase::*)( vector<string>&, File& )>::iterator fileInfoVectorIter;
	
	Panel *panel;   
	QuestionManager *questionManager; 
	Settings &settings;
	
	bool VfsTypeLocal;
	Mime mime;

	
	


    
    
    

};

	template<class T>
	void VfsBase::goRecursive(File & file, T & operation,bool visitChildrenFirst,Cancelable *cancelable)
	{
	
	
		if(cancelable && cancelable->isCanceledExecution())
		return;
	
		
		bool isDirectory=false;
	
		if(file.isDirectory() && !file.isSymlink() && file.normal())
		isDirectory=true;
		
		if(!visitChildrenFirst)
		operation.process(file);
		
		if (isDirectory)
		{
			path p=file.getPath();
			Directory directory=openDir(p);
			
			try
			{
				while(1)
				{
				
					File nextFile= directory.getNextFile();
					if(!nextFile.normal())
					continue;
					
					goRecursive(nextFile,operation,visitChildrenFirst,cancelable);	
				}
			}
			catch(exception e)
			{
			}
				
		}
		
		if(visitChildrenFirst)
		operation.process(file);
		
	}


#endif
