/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "VfsSync.hpp"

VfsSync::VfsSync(Vfs * _vfs): vfs(_vfs)
{
}

Directory VfsSync::openDir(path &p, Cancelable *cancelable)
{
	boost::mutex::scoped_lock lock(muteX);
	return vfs->openDir(p,cancelable);
}

double VfsSync::getTotalSize(path & p, Cancelable *cancelable)
{
	boost::mutex::scoped_lock lock(muteX);
	return vfs->getTotalSize(p,cancelable);
}


void VfsSync::createDir(path p)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->createDir(p);
}

File VfsSync::readFile(path file)
{
	boost::mutex::scoped_lock lock(muteX);
	return vfs->readFile(file);
}


void VfsSync::copyToFile(File &source, path & destination, Status * status, Cancelable *cancelable)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->copyToFile(source,destination,status,cancelable);
}

void VfsSync::copyFromFile(File &source, path & destination, Status * status, Cancelable *cancelable)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->copyFromFile(source,destination,status,cancelable);
}

void VfsSync::closeDir(Directory * directory)
{
	boost::mutex::scoped_lock lock(muteX);
	return vfs->closeDir(directory);
}

File VfsSync::readDir(Directory * directory)
{
	boost::mutex::scoped_lock lock(muteX);
	return vfs->readDir(directory);
}


string VfsSync::getType()
{
	boost::mutex::scoped_lock lock(muteX);
	return vfs->getType();
}

path VfsSync::symlink(const path & file)
{
	boost::mutex::scoped_lock lock(muteX);
	return vfs->symlink(file);
}

void VfsSync::init(Panel * panel,QuestionManager *questionManager)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->init(panel,questionManager);
}



void VfsSync::chmod(path & source, unsigned int mode)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->chmod(source,mode);
}

void VfsSync::chmodRecursive(path & source, unsigned int mode)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->chmod(source,mode);


}

void VfsSync::removeFile(path & source)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->removeFile(source);
}

void VfsSync::removeDirectory(path & source)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->removeDirectory(source);
}

void VfsSync::remove(path & source)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->remove(source);
}

void VfsSync::rename(path & source, path & destination)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->rename(source,destination);
}


void VfsSync::copy(Vfs * vfs2, path source, path destination, Status * status,Cancelable *cancelable)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->copy(vfs2,source,destination,status,cancelable);
}

File VfsSync::readFullFile(File file)
{
	boost::mutex::scoped_lock lock(muteX);
	return vfs->readFullFile(file);
}




void VfsSync::changeOwnerRecursive(path & source, string user)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->changeOwnerRecursive(source,user);
}

void VfsSync::changeOwner(path & source, string user)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->changeOwner(source,user);
}

vector< string > VfsSync::getGroups()
{
	boost::mutex::scoped_lock lock(muteX);
	return vfs->getGroups();
}

vector< string > VfsSync::getUsers()
{
	boost::mutex::scoped_lock lock(muteX);
	return vfs->getUsers();
}

void VfsSync::changeGroup(path & source, string group)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->changeGroup(source,group);
}

void VfsSync::changeGroupRecursive(path & source, string group)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->changeGroupRecursive(source,group);
}

void VfsSync::createSymlink(path &source,path &destination)
{
	boost::mutex::scoped_lock lock(muteX);
	vfs->createSymlink(source,destination);

}
