/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "XmlReaderWriter.hpp"
#include "Exceptions.hpp"


XmlReaderWriter::XmlReaderWriter(bool _save)
{
	doc = NULL;
	save=_save;
}


XmlReaderWriter::~XmlReaderWriter()
{

	if ( save && !copy && doc)
	{			

		xmlSaveFormatFileEnc( xmlFileName.string().c_str(), doc, "UTF-8", 1 );
		std::string cmd = "xmllint --format " + xmlFileName.string() + " -o " + xmlFileName.string();
            	system( cmd.c_str() );
		xmlFreeDoc ( doc );
		xmlCleanupParser ();

	}
}

XmlReaderWriter::XmlReaderWriter( XmlReaderWriter & conf )
{
    doc = conf.doc;
    copy = true;
}



void XmlReaderWriter::loadXml( path _xmlFileName )
{
	xmlFileName=_xmlFileName;

	xmlKeepBlanksDefault(0);
	doc = xmlParseFile ( xmlFileName.string().c_str () );
	
	if(!doc)
	throw ExceptionReadFile();
   

}

string XmlReaderWriter::read( string name )
{

	xmlChar * xpath = ( xmlChar * ) name.c_str ();
	xmlChar *keyword;
	
	string retstring;	
	
	xmlNodeSetPtr nodeset;
	xmlXPathContextPtr context;
	xmlXPathObjectPtr result;
	
	context = xmlXPathNewContext ( doc );
	if ( context == NULL )
		return "";
	
	result = xmlXPathEvalExpression ( xpath, context );
	xmlXPathFreeContext ( context );
	if ( result == NULL )
		return "";
	
	
	if ( xmlXPathNodeSetIsEmpty ( result->nodesetval ) )
	{
		return "";
	}
		
	nodeset = result->nodesetval;
	keyword = xmlNodeListGetString ( doc, nodeset->nodeTab[ 0 ] ->xmlChildrenNode, 1 );
	if ( keyword != NULL )
		retstring = string ( ( char * ) keyword );
	xmlFree ( keyword );
	
	xmlXPathFreeObject ( result );
	
	return retstring;

}

bool XmlReaderWriter:: openPath ( string path )
{
    xmlChar * xpath = ( xmlChar * ) path.c_str ();
    xmlXPathContextPtr context;

    context = xmlXPathNewContext ( doc );
    if ( context == NULL )
        return false;

    result = xmlXPathEvalExpression ( xpath, context );
    xmlXPathFreeContext ( context );
    if ( result == NULL )
        return false;

    if ( xmlXPathNodeSetIsEmpty ( result->nodesetval ) )
    {
        return false;
    }
    nodeset = result->nodesetval;
    pos = 0;
    return true;

}

bool XmlReaderWriter::readNode ( map<string,string> &node )
{

	if(pos < nodeset->nodeNr)
	{
		cur = nodeset->nodeTab[ pos++ ];
		xmlNodePtr nod=cur->children;
			
			while(nod)
			{
				string name=( char * )nod->name;
				string value;
				xmlChar *keyword = xmlNodeListGetString ( doc, nod->children, 1 );
					if ( keyword != NULL )
					{
					value = string ( ( char * ) keyword );
					xmlFree ( keyword );
					}
				node[name]=value;
			nod=nod->next;						
			}
	}
	else
	{
        	xmlXPathFreeObject ( result );
		return false;
	}

return true;
}

bool XmlReaderWriter::readNodeVector ( map<string,vector<string> > &node )
{

	if(pos < nodeset->nodeNr)
	{
		cur = nodeset->nodeTab[ pos++ ];
		xmlNodePtr nod=cur->children;
			
			while(nod)
			{
				string name=( char * )nod->name;
				string value;
				xmlChar *keyword = xmlNodeListGetString ( doc, nod->children, 1 );
					if ( keyword != NULL )
					{
					value = string ( ( char * ) keyword );
					xmlFree ( keyword );
					}
				node[name].push_back(value);
			nod=nod->next;						
			}
	}
	else
	{
        	xmlXPathFreeObject ( result );
		return false;
	}

return true;
}


bool XmlReaderWriter::readNodeString( string & name )
{

	if(pos < nodeset->nodeNr)
	{
		cur = nodeset->nodeTab[ pos++ ];
		xmlChar *keyword = xmlNodeListGetString ( doc, cur->children, 1 );
		if ( keyword != NULL )
		{
		name = string ( ( char * ) keyword );
		xmlFree ( keyword );
		}	
	}
	else
	{
		xmlXPathFreeObject ( result );
		return false;
	}

return true;
}


bool XmlReaderWriter::addNodeString ( string path, string node, string value )
{
	//wstring value(_value.begin(), _value.end());

    xmlChar * xpath = ( xmlChar * ) path.c_str ();
    xmlChar *keyword;
    std::string retstring;


    xmlNodeSetPtr nodeset;
    xmlXPathContextPtr context;
    xmlXPathObjectPtr result;

    context = xmlXPathNewContext ( doc );
    if ( context == NULL )
        return false;

    result = xmlXPathEvalExpression ( xpath, context );
    xmlXPathFreeContext ( context );
    if ( result == NULL )
        return false;


    if ( xmlXPathNodeSetIsEmpty ( result->nodesetval ) )
    {
        return false;
    }

    nodeset = result->nodesetval;
    if(xmlCheckUTF8(( xmlChar * )value.c_str()))
    xmlNewChild ( nodeset->nodeTab[ 0 ], NULL, ( xmlChar * ) node.c_str (), ( xmlChar * ) value.c_str () );

 
    xmlXPathFreeObject ( result ); 
    return true;

}

bool XmlReaderWriter::clear( )
{
    xmlChar * xpath = (xmlChar *) "/NaoConfig";
    xmlChar *keyword;
    std::string retstring;

    xmlNodeSetPtr nodeset;
    xmlXPathContextPtr context;
    xmlXPathObjectPtr result;

    context = xmlXPathNewContext ( doc );
    if ( context == NULL )
        return false;

    result = xmlXPathEvalExpression ( xpath, context );
    xmlXPathFreeContext ( context );
    if ( result == NULL )
        return false;

    if ( xmlXPathNodeSetIsEmpty ( result->nodesetval ) )
    {
        return false;
    }

    nodeset = result->nodesetval;
    xmlUnlinkNode ( nodeset->nodeTab[ 0 ] );
    xmlFreeNode ( nodeset->nodeTab[ 0 ] );
    xmlXPathFreeObject ( result );
    
    return true;
}

bool XmlReaderWriter::writeNode( string path,string name, map< string, string >  node )
{
    xmlChar * xpath = ( xmlChar * ) path.c_str ();
    xmlChar *keyword;
    std::string retstring;


    xmlNodeSetPtr nodeset;
    xmlXPathContextPtr context;
    xmlXPathObjectPtr result;

    context = xmlXPathNewContext ( doc );
    if ( context == NULL )
        return false;

    result = xmlXPathEvalExpression ( xpath, context );
    xmlXPathFreeContext ( context );
    if ( result == NULL )
        return false;


    if ( xmlXPathNodeSetIsEmpty ( result->nodesetval ) )
    {
        return false;
    }

    nodeset = result->nodesetval;
    xmlNodePtr child=xmlNewChild ( nodeset->nodeTab[ 0 ], NULL, ( xmlChar * ) name.c_str (), NULL );
    
    	map< string, string >::iterator iter;
   	for(iter=node.begin();iter!=node.end();++iter)
    	{
    		if(iter->second.size())
		xmlNewChild ( child, NULL, ( xmlChar * )iter->first.c_str (), ( xmlChar * )iter->second.c_str () );
	}
 
    xmlXPathFreeObject ( result ); 
    return true;
	
	
}

bool XmlReaderWriter::writeNodeVector( string path,string name, map< string, vector<string> >  node )
{
    xmlChar * xpath = ( xmlChar * ) path.c_str ();
    xmlChar *keyword;
    std::string retstring;


    xmlNodeSetPtr nodeset;
    xmlXPathContextPtr context;
    xmlXPathObjectPtr result;

    context = xmlXPathNewContext ( doc );
    if ( context == NULL )
        return false;

    result = xmlXPathEvalExpression ( xpath, context );
    xmlXPathFreeContext ( context );
    if ( result == NULL )
        return false;


    if ( xmlXPathNodeSetIsEmpty ( result->nodesetval ) )
    {
        return false;
    }

    nodeset = result->nodesetval;
    xmlNodePtr child=xmlNewChild ( nodeset->nodeTab[ 0 ], NULL, ( xmlChar * ) name.c_str (), NULL );
    
    map< string, vector<string> >::iterator iter;
    	for(iter=node.begin();iter!=node.end();++iter)
    	{
    		vector<string>::iterator vecIter;
    		for(vecIter=iter->second.begin();vecIter!=iter->second.end();++vecIter)
    		{
    			if(vecIter->size())
			xmlNewChild ( child, NULL, ( xmlChar * )iter->first.c_str (), ( xmlChar * )vecIter->c_str () );
		}
	}
 
    xmlXPathFreeObject ( result ); 
    return true;
	
	
}

