/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "FoxFileListPanel.hpp"
#include "Iconv.hpp"


#include <boost/filesystem/path.hpp>

using namespace boost::filesystem;


FXDEFMAP ( FoxFileListPanel ) FoxFileListPanelMap[] =
    {
        //________Message_Type_____________________ID____________Message_Handler_______
        FXMAPFUNCS ( SEL_COMMAND, FoxFileListPanel::ID_PATHBUTTON, FoxFileListPanel::ID_PATHROOTBUTTON,FoxFileListPanel::onPathButtonOpen ),
    	FXMAPFUNCS ( SEL_RIGHTBUTTONRELEASE, FoxFileListPanel::ID_PATHBUTTON, FoxFileListPanel::ID_PATHROOTBUTTON, FoxFileListPanel::onPathButtonOpen ),
    	FXMAPFUNC ( SEL_COMMAND, FoxFileListPanel::ID_OPENDIR, FoxFileListPanel::onOpenDir ),
    	
    };

FXIMPLEMENT ( FoxFileListPanel, FXObject, FoxFileListPanelMap, ARRAYNUMBER ( FoxFileListPanelMap ) )



FoxFileListPanel::FoxFileListPanel(FoxPanelManager *foxPanelManager,string param)
 : FoxPanel(foxPanelManager,param),directoriesMenu(NULL),list(false),enabled(true)
{
	p_id=pthread_self();	
	guiSignal= new FXGUISignal (foxPanelManager->getLeftcontrolframe()->getApp(),&foxSingalReceiver);
	
	FoxPanel::setType( "FoxFileListPanel" );
	constructor=true;
	foxFileList=new FoxFileList(frame,foxPanelManager->getDock(),this);
	constructor=false;
	foxFileList->hideToolbars();
	
	
	
	
}

FoxFileListPanel::~FoxFileListPanel()
{
	delete guiSignal;
	
	if ( directoriesMenu )
	delete directoriesMenu;
}

void FoxFileListPanel::setPath( string dir )
{
	prepare<void>(boost::bind(&FoxFileListPanel::setPathSync, this, dir),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}


void FoxFileListPanel::setPathSync( string dir )
{
enabled=true;
currentDirectory=dir;

	for(buttonsIterator=buttons.begin();buttonsIterator!=buttons.end();++buttonsIterator)
	{
		delete *buttonsIterator;
	}
	buttons.clear();

	path p=dir;
	path::iterator iter;

	for( iter=p.begin();iter!=p.end() ;++iter )
	{
		string field;
		if(dir=="/")
		field="/";
		else if(iter==p.begin())
		continue;
		else
		field="/"+string(iter->c_str());
		
		if(!Iconv::isUTF8(field))
		{		
			field=Iconv::toUTF8(field);
		}
		
		FXButton*button=new FXButton ( hf, field.c_str(), NULL, this, FoxFileListPanel::ID_PATHBUTTON, FRAME_THICK, 0, 0, 0, 0, 0, 0, 0, 0 );
		buttons.push_back(button);
		
		/*UGLY :(*/
		if(!constructor)
		button->create();
		
		button->setUserData(this);
	}
	
	hf->recalc();
	
	if(list)
	{
	listPath(dir);
	list=false;
	}


}

void FoxFileListPanel::openPath( string p )
{
	foxFileList->openPath( p );
}


void FoxFileListPanel::listPath(string path)
{
	directoriesMap.clear();
	FoxIconManager &foxIconManager=FoxSingleton<FoxIconManager>::instance( hf->getApp());	
		
		vector<string>directories=foxFileList->getDirectories( );
		vector<string>::iterator iter;
		
		if ( directories.size () > 0 )
    		{
        		sort ( directories.begin (), directories.end () );
        		if ( directoriesMenu )
        		{
				directoriesMenu->popdown ();
				delete directoriesMenu;
				directoriesMenu = NULL;
			}
			
			if(directories.size()<=20)
       			directoriesMenu = new FXMenuPane ( hf->getRoot() );     
       			else  				
       			directoriesMenu=new FXScrollPane(hf->getRoot(),20,PACK_UNIFORM_HEIGHT);
		
			for(iter=directories.begin();iter!=directories.end();++iter)
			{
				string name=iter->c_str ();
				if(!Iconv::isUTF8(iter->c_str ()))
				{
					name=Iconv::toUTF8(iter->c_str ());
				}
			
 				new FXMenuCommand ( directoriesMenu, name.c_str(), foxIconManager.get("folder"), this, FoxFileListPanel::ID_OPENDIR );
				directoriesMap[name]=iter->c_str ();
			}
			
			directoriesMenu->create ();
			directoriesMenu->popup ( NULL, x, y, 0, 0 );
        	//button->getX()
        	}


}



long FoxFileListPanel::onPathButtonOpen( FXObject * sender, FXSelector sel , void *ptr )
{
	if(!enabled)
	return 0;

	FXEvent *event = ( FXEvent * ) ptr;
	
	FXButton * button = ( FXButton * ) sender;
	list=false;
	if(FXSELTYPE(sel)==SEL_RIGHTBUTTONRELEASE)
	{
	list=true;
	x=event->root_x;
	y=event->root_y;
	}
	
	if(list!=true && FXSELID(sel)==ID_PATHROOTBUTTON)
	{
	enabled=false;	
	openPath("/");
	return 0;
	}
	
	if(list==true && FXSELID(sel)==ID_PATHROOTBUTTON)
	{
	listPath("/");
	return 0;
	}
	
	string path;

	for(buttonsIterator=buttons.begin();buttonsIterator!=buttons.end();++buttonsIterator)
	{
		path+=string((*buttonsIterator)->getText().text());
		if(*buttonsIterator==button)
		{		
		break;
		}
	}
	
	if(++buttonsIterator==buttons.end() && list==true)
	{
		listPath(currentDirectory);	
	}
	else	
	{
	enabled=false;	
	openPath(path);
	}
	

}

long FoxFileListPanel::onOpenDir( FXObject * sender, FXSelector , void * )
{
	FXMenuCommand * bt = ( FXMenuCommand * ) sender;
	string name=bt->getText().text();
	
	name=directoriesMap[name];
	directoriesMap.clear();
	
	directoriesMenu->popdown ();
	delete directoriesMenu;
	directoriesMenu = NULL;
	
	string path;
	if(currentDirectory=="/")
	path="/"+name;
	else
	path=currentDirectory +"/"+ name;
	
	list=true;
	enabled=false;
	foxFileList->openPath( path );
	//listPath(path);

}

void FoxFileListPanel::moveToRight( void )
{
	FoxPanel::moveToRight();
	foxFileList->lostFocus();
}

void FoxFileListPanel::moveToLeft( void )
{
	FoxPanel::moveToLeft();
	foxFileList->lostFocus();
}

void FoxFileListPanel::moveToTop( void )
{
	//foxFileList->hideToolbars();
	//foxFileList->killFocus();
	FoxPanel::moveToTop();
	foxFileList->lostFocus();
}


void FoxFileListPanel::setOpposite(FoxPanel* theValue)
{
    opposite = theValue;
    

    
    if(opposite && opposite->getType()=="FoxFileListPanel")
    {
    	FoxFileListPanel *opp=dynamic_cast<FoxFileListPanel*>(opposite);
    	foxFileList->setOpposite( opp->getFoxFileList() );
    }
    else
    {
    	foxFileList->setOpposite( NULL );
    }

}

void FoxFileListPanel::directoryLoaded()
{

}

void FoxFileListPanel::enable()
{
	//enabled=true;
}

void FoxFileListPanel::disable()
{
	//enabled=false;
}



bool FoxFileListPanel::canClose()
{
	return !foxFileList->isProcessing();
}

void FoxFileListPanel::setLabel(string text, string iconName)
{
	label->setSelector(ID_PATHROOTBUTTON);
	label->setTarget(this);
	FoxPanel::setLabel(text,iconName);
}

void FoxFileListPanel::setFocus(void )
{
	foxFileList->setFocusSync();
}

void FoxFileListPanel::createPanel(string name, string param)
{
	prepare<void>(boost::bind(&FoxFileListPanel::createPanelSync, this, name,param),p_id,guiSignal,FoxSingalReceiver::ID_GUISIGNAL,mutexS);
}

void FoxFileListPanel::createPanelSync(string name, string param)
{
	foxPanelManager->createPanel(name,param);
}

