/***************************************************************************
 *   Copyright (C) 2007 by Mateusz Dworak aka compbatant                   *
 *   compbatant@linux.pl                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef VFSFTP_HPP
#define VFSFTP_HPP

#include "VfsBase.hpp"
#include "Settings.hpp"
#include "Mime.hpp"
#include "../libfactory/ClassFactory.hpp"
#include "../libfactory/Factorable.hpp"

#include "ftplib.h"


/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/
class VfsFtp : public VfsBase, public Factorable
{
public:
	VfsFtp();
	~VfsFtp();
	
	Directory openDir( path &p, Cancelable *cancelable=NULL  );
	
	void createDir(path p);
	File readFile(path file);
  
    	void copyToFile(File &source, path &destination, Status * status, Cancelable *cancelable);
    	void copyFromFile(File &source, path &destination, Status * status, Cancelable *cancelable);
    	string getType();
    	void init(Panel *panel,QuestionManager *questionManager);
    	void chmod(path &source,unsigned int mode);
    	void removeFile(path &source);
    	void removeDirectory(path &source);
    	void rename(path &source, path &destination);
    	void changeOwner(path &source,string user);
    	void changeGroup(path &source,string group);
    	
private:
    	
    	friend class Directory;
    	
    	File readDir( Directory *directory);
    	void closeDir(Directory *directory);
    	
    	Mime mime;
    	map<string,string>params;
    	static const int BLOCK_SIZE=131072;//128KB
    	   	
    	bool connected;
    	string connectionDescription;
    	
    	
    	void connect();
    	ftplib ftp;
    	
    	
	
};

#endif
