#ifndef VFSSFTP_HPP
#define VFSSFTP_HPP

#include "VfsBase.hpp"
#include "Settings.hpp"
#include "Mime.hpp"
#include "FilesEngine.hpp"
#include "Directory.hpp"

#include "../libfactory/ClassFactory.hpp"
#include "../libfactory/Factorable.hpp"

#include <libssh/libssh.h>
#include <libssh/sftp.h>

/**
	@author Mateusz Dworak aka compbatant <m@nao.linux.pl>
*/



class VfsSftp : public Factorable, public VfsBase
{
public:
    VfsSftp();
    ~VfsSftp();
    
	Directory openDir( path &p, Cancelable *cancelable=NULL  );
	
	void createDir(path p);
	File readFile(path file);
  
    	void copyToFile(File &source, path &destination, Status * status, Cancelable *cancelable);
    	void copyFromFile(File &source, path &destination, Status * status, Cancelable *cancelable);
    	string getType();
    	void init(Panel *panel,QuestionManager *questionManager);
    	void chmod(path &source,unsigned int mode);
    	void removeFile(path &source);
    	void removeDirectory(path &source);
    	void rename(path &source, path &destination);
    	void changeOwner(path &source,string user);
    	void changeGroup(path &source,string group);
    	
    	
private:
    
    	
	void connect (void);
	void disconnect(void);
	File readRFile(SFTP_ATTRIBUTES *rfile,path parentDir,string name);
	void setFileSize(vector<string>&fields, File& file);
	void setFileMime(vector<string>&fields, File& file);
	void translateErrnoToException(string fileName, string operation);
	string getHex(unsigned char *what, int len);
	
	string connectionDescription;
	bool connected;
	
	SSH_SESSION *session;
	SFTP_SESSION *sftp;
	SFTP_DIR *dirsftp;
	vector<void (VfsSftp::*)( vector<string>&, File& )> fileInfoVector;
	vector<void (VfsSftp::*)( vector<string>&, File& )>::iterator fileInfoVectorIter;
		
 	Mime mime;
 	static const int BLOCK_SIZE=131072;//128KB
    	map<string,string>params;
    	
    	
    	friend class Directory;
    	
    	File readDir( Directory *directory);
    	void closeDir(Directory *directory);
    	
};

#endif
